package com.bytesw.bytebot.controller;

import com.bytesw.bytebot.etl.beans.GoalBean;
import com.bytesw.bytebot.service.GoalService;
import com.bytesw.xdf.annotation.ProgramSecurity;
import com.bytesw.xdf.controller.XDFController;
import com.google.gson.GsonBuilder;
import io.swagger.annotations.ApiParam;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@RestController()
@RequestMapping("/service/settings/goal")
@ProgramSecurity("GOAL")
@Log4j2
public class GoalController extends XDFController<GoalBean, Long> {
    @Autowired
    private GoalService goalService;

    @Autowired
    private GsonBuilder gsonBuilder;

    public GoalController(GoalService service) {
        super(service);
    }

    @PostMapping(value = "/goalForAction")
    @PreAuthorize("hasPermission(this, 'view')")
    public ResponseEntity<String> createGoalForAction(@RequestBody Map<String, Object> body) {
        GoalBean info = goalService.saveGoalForAction(body);
        return new ResponseEntity(gson.toJson(info), HttpStatus.OK);
    }

    @DeleteMapping(value = "/goalForAction/{id}")
    @PreAuthorize("hasPermission(this, 'view')")
    public ResponseEntity<String> GoalForAction(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
        String info = "";
        try{
            goalService.deleteGoalForAction(id);
            info = id.toString();
            return new ResponseEntity(gson.toJson(info), HttpStatus.OK);
        } catch (Exception e) {
            info = e.getMessage();
        }
        return new ResponseEntity(gson.toJson(info), HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @PutMapping(value = "/goalForAction/{id}")
    @PreAuthorize("hasPermission(this, 'edit')")
    public ResponseEntity<String> updateConversationalAgent(@PathVariable("id") Long id,
                                                            @RequestBody Map<String, Object> body) {
        String info = "";
        try{
            goalService.updateGoalForActions(body, id);
            return new ResponseEntity(gson.toJson(info), HttpStatus.OK);
        } catch (Exception e) {
            info = e.getMessage();
        }
        return new ResponseEntity(gson.toJson(info), HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @GetMapping(value = "/goalForAction/{id}")
    @PreAuthorize("hasPermission(this, 'view')")
    public ResponseEntity<String> getActionByGoalId(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
        HttpStatus hs = HttpStatus.OK;
        try {
            return new ResponseEntity<>(gsonBuilder.create().toJson(goalService.getGoalForActions(id)), hs);
        } catch (Exception e) {
            String info = "Error detectado al obtener informacion";
            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(info), hs);
        }
    }

    @GetMapping(value = "/goalForAction/")
    @PreAuthorize("hasPermission(this, 'view')")
    public ResponseEntity<String> getActionByGoalIdAll() {
        HttpStatus hs = HttpStatus.OK;
        try {
            return new ResponseEntity<>(gsonBuilder.create().toJson(goalService.getGoalForActionsAll()), hs);
        } catch (Exception e) {
            String info = "Error detectado al obtener informacion";
            hs = HttpStatus.INTERNAL_SERVER_ERROR;
            return new ResponseEntity<>(gsonBuilder.create().toJson(info), hs);
        }
    }
}
